/*
 * Copyright (C) 2005-2006 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */
#include "para.h"

#include "recv.h"
#include "recv.cmdline.h"
#include "fd.h"
#include "error.h"

struct gengetopt_args_info conf;

INIT_RECV_ERRLISTS;

__printf_2_3 void para_log(int ll, const char* fmt,...)
{
	va_list argp;

	/* ignore log message if loglevel is not high enough */
	if (ll < conf.loglevel_arg)
		return;
	va_start(argp, fmt);
	vfprintf(stderr, fmt, argp);
	va_end(argp);
}

static void *parse_config(int argc, char *argv[], int *receiver_num)
{
	int i;

	if (cmdline_parser(argc, argv, &conf))
		return NULL;
	if (conf.list_receivers_given) {
		printf("available receivers: ");
		for (i = 0; receivers[i].name; i++)
			printf("%s%s", i? " " : "", receivers[i].name);
		printf("\nTry\n\tpara_recv -r '<receivername> -h'\n"
			"for help on <receivername>.\n");
		exit(EXIT_SUCCESS);
	}
	return check_receiver_arg(conf.receiver_arg, receiver_num);
}

int main(int argc, char *argv[])
{
	int ret, eof = 0, max, r_opened = 0, receiver_num;
	struct timeval timeout;
	struct	receiver *r = NULL;
	fd_set rfds, wfds;
	struct receiver_node rn;

	memset(&rn, 0, sizeof(struct receiver_node));
	for (ret = 0; receivers[ret].name; ret++)
		receivers[ret].init(&receivers[ret]);
	ret = -E_RECV_SYNTAX;
	rn.conf = parse_config(argc, argv, &receiver_num);
	if (!rn.conf) {
		PARA_EMERG_LOG("%s", "parse failed\n");
		goto out;
	}
	r = &receivers[receiver_num];
	rn.receiver = r;
	ret = r->open(&rn);
	if (ret < 0)
		goto out;
	r_opened = 1;
recv:
	FD_ZERO(&rfds);
	FD_ZERO(&wfds);
	timeout.tv_sec = 0;
	timeout.tv_usec = 999 * 1000;
	max = -1;
	ret = r->pre_select(&rn, &rfds, &wfds, &timeout);
	max = PARA_MAX(max, ret);

	PARA_DEBUG_LOG("timeout: %lums, max: %d\n", tv2ms(&timeout), max);
	ret = para_select(max + 1, &rfds, &wfds, &timeout);
	if (ret < 0) {
		ret = -E_RECV_SELECT;
		goto out;
	}
	ret = r->post_select(&rn, ret, &rfds, &wfds);
	if (ret < 0)
		goto out;
	if (!ret)
		eof = 1;
	if (!rn.loaded) {
		if (eof)
			goto out;
		goto recv;
	}
	ret = write(STDOUT_FILENO, rn.buf, rn.loaded);
	PARA_DEBUG_LOG("wrote %d/%zd\n", ret, rn.loaded);
	if (ret < 0) {
		ret = -E_WRITE_STDOUT;
		goto out;
	}
	if (ret != rn.loaded) {
		PARA_INFO_LOG("short write %d/%zd\n", ret, rn.loaded);
		memmove(rn.buf, rn.buf + ret, rn.loaded - ret);
	}
	rn.loaded -= ret;
	if (rn.loaded || !eof)
		goto recv;
out:
	if (r_opened)
		r->close(&rn);
	if (r)
		r->shutdown();
	if (ret < 0)
		PARA_ERROR_LOG("%s\n", PARA_STRERROR(-ret));
	return ret;
}
